/**
 * Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ project (https://gitlab.com/desq/)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

#include "Global.hpp"
#include "Table.hpp"
#include <wlrootsqt/DataControl.hpp>

namespace DesQ {
    namespace Clipboard {
        class Manager;
    }
}

class DesQ::Clipboard::Manager : public QObject {
    Q_OBJECT;

    public:
        Manager();

        /**
         * Request the manager to offer data at index @idx
         * from ClipboardType @type. The manager is free to
         * deny this reuest.
         *
         * Accepting this request entails the data at index @idx
         * to be promoted to top. This will require the UIs to
         * be reloaded.
         */
        void requestOffer( DesQ::Clipboard::Type type, int idx );

    private:
        /** DataControlManager pointer; Used multiple times */
        WlrootsQt::DataControlManager *mDataMgr;

        /** DataControlDevice pointer; Used multiple times */
        WlrootsQt::DataControlDevice *mDataDev;

        /** Clipboard table */
        DesQ::Clipboard::Table *clipboard;

        /** Selection table */
        DesQ::Clipboard::Table *selection;

        /** Stores the last gathered Clipboard data which is not null */
        WlrootsQt::MimeData lastClipboard;

        /** Stores the last gathered Primary data which is not null */
        WlrootsQt::MimeData lastSelection;

        /** For testing purposes */
        WlrootsQt::DataControlSource *clpSrc;
        WlrootsQt::DataControlSource *selSrc;

        /**
         * Clipboard offer - Store the data if not null.
         * Then, offer the last stored data.
         */
        void handleClipboardOffer( WlrootsQt::DataControlOffer *offer );

        /**
         * Selection offer - Store the data if not null.
         * Then, offer the last stored data.
         */
        void handleSelectionOffer( WlrootsQt::DataControlOffer *offer );

        /**
         * Offer the last collected data as Clipboard
         */
        void offerClipboard( WlrootsQt::MimeData mData );

        /**
         * Selection offer - Store the data if not null.
         * Then, offer the last stored data.
         */
        void offerSelection( WlrootsQt::MimeData mData );

        /**
         * Gather the offered data
         */
        WlrootsQt::MimeData gatherOfferedData( WlrootsQt::DataControlOffer *offer );

    Q_SIGNALS:
        void updateView( int );
        void largeData( int );
};
